# Generated by Django 4.2.10 on 2024-04-25 08:57

from typing import List

from django.db import migrations


class Migration(migrations.Migration):
    dependencies = [
        ("database", "0037_chatmodeloptions_openai_config_and_more"),
        ("database", "0037_searchmodelconfig_bi_encoder_docs_encode_config_and_more"),
    ]

    operations: List[str] = []
